"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

"""
Django settings for sample_app project.

Generated by 'django-admin startproject' using Django 1.10.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '+jnvf=z69!l@j(^*^h*+fdlt_61!1v446znccb)9%@tjiw1@x-'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'django_tequila',
    'sample_app',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django_tequila.middleware.TequilaMiddleware',
]

ROOT_URLCONF = 'sample_app.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'sample_app.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'sample_app', 'database.db'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'

# Django-tequila specifics

AUTHENTICATION_BACKENDS = ('django_tequila.django_backend.TequilaBackend',)
TEQUILA_SERVICE_NAME = "django_tequila_service"
TEQUILA_SERVER_URL = "https://tequila.epfl.ch"
TEQUILA_NEW_USER_INACTIVE = False
TEQUILA_CLEAN_URL = True
TEQUILA_STRONG_AUTHENTICATION = False
LOGIN_URL = "/login"
LOGIN_REDIRECT_URL = "/"
LOGOUT_URL = "/"
LOGIN_REDIRECT_IF_NOT_ALLOWED = "/not_allowed"
LOGIN_REDIRECT_TEXT_IF_NOT_ALLOWED = "Not allowed : please contact your admin"

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        'django_tequila': {
            'handlers': ['console'],
            'level': 'DEBUG',
        },
    },
}



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

import json
import logging
import sys
import urllib

logger = logging.getLogger('django_tequila.client')

from django.utils.encoding import smart_str

try:
    # For Python 2
    from urllib2 import Request
except ImportError:
    # For Python 3
    from urllib.request import Request

try:
    # For Python 2
    url_encoding_method = urllib.urlencode
except AttributeError:
    # For Python 3
    url_encoding_method = urllib.parse.urlencode

try:
    # For Python 2
    from urllib2 import urlopen
except ImportError:
    # For Python 3
    from urllib.request import urlopen


class TequilaClient(object):
    def __init__(self, config):
        self.config = config

        self.is_authenticated = False
        self.request_key = None

    def _open_url(self, url, data=None, additional_values=""):
        """ hide urllib2 access, return header and response """
        url_values = None

        if data:
            url_values = url_encoding_method(data)
        if url_values:
            final_url = url + "?" + url_values + additional_values
        else:
            final_url = url

        req = Request(final_url)

        # TODO: try..except HTTPError or URLError
        logger.debug("Opening url %s..." % final_url)
        response = urlopen(req)

        return response.read()

    def _get_key(self):
        """ hide urllib2 access """
        logger.debug("Asking for a key...")

        if self.request_key:
            logger.debug("Key already loaded...")
            return self.request_key

        logger.debug("No key found, fetching a new one...")

        if self.config.request:
            list_request = '+'.join(self.config.request)
        else:
            raise ValueError("username attribute is mandatory in request")

        params = {'urlacces': smart_str(self.config.redirect_to),
                  'service': self.config.service,
                  'allows': self.config.allows
                  }

        if self.config.additional_params:
            params.update(self.config.additional_params)

        self.request_key = self._open_url(
            self.config.server_url + "/cgi-bin/tequila/createrequest", params,
            "&request=" + list_request)[4:-1]

        logger.debug("Key generated : %s " % self.request_key)

        return self.request_key

    key = property(_get_key)

    def login_url(self):

        if sys.version_info < (3,):
            key = self.key
        else:
            key = self.key.decode('UTF-8')
        return self.config.server_url + "/cgi-bin/tequila/auth?requestkey=" + key

    def get_attributes(self, key=None):
        """ return a dictionnary of attributes setted by tequila,
            corresponding with the "request" parameter in config
         """
        logger.debug("Asking for attributes...")

        if key:
            params = {'key': key}
        elif self.request_key:
            params = {'key': self._get_key()}
        else:
            raise ValueError()

        response = self._open_url(
            self.config.server_url + "/cgi-bin/tequila/fetchattributes",
            params)

        attributes = {}

        # force utf-8 as django need it
        response = response.decode('utf-8')

        list_attributes = response.split('\n')

        logger.debug(
            "Attributes returned by %s : \n%s" % (self.config.server_url,
                                                  json.dumps(list_attributes,
                                                             sort_keys=True,
                                                             indent=4)))

        for attribute in list_attributes[:-1]:
            splitted_attr = attribute.split('=')

            if len(splitted_attr) == 1:
                splitted_attr.append('')

            attributes[splitted_attr[0]] = splitted_attr[1]

        if not self._verify_attributes(attributes):
            raise StandardError()

        logger.debug(
            "Attributes after formatting and returned: \n%s" % json.dumps(
                attributes,
                sort_keys=True, indent=4))

        return attributes

    def _verify_attributes(self, attributes):
        needed_attrs = ['org', 'user', 'host', 'key']

        for needed_attribute in needed_attrs:
            try:
                if not attributes[needed_attribute]:
                    return False
            except KeyError:
                return False
        return True



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""


class Config(object):
    def __init__(self,
                 server_url,
                 additional_params=None,
                 redirect_to=None,
                 service=None,
                 request=None,
                 language=None,
                 localserver=None,
                 org=None,
                 langage=None,
                 want_right=None,
                 want_role=None,
                 allows=None,
                 strong_authentication=False
                 ):

        self.server_url = server_url

        if redirect_to:
            if redirect_to.find('http://') == -1 and redirect_to.find('https://') == -1:
                self.redirect_to = 'https://' + redirect_to
            else:
                self.redirect_to = redirect_to
        else:
            self.redirect_to = None

        self.additional_params = additional_params

        if strong_authentication:
            strong_authentication_param = {'authstrength': 3}

            if self.additional_params:
                self.additional_params.update(strong_authentication_param)
            else:
                self.additional_params = strong_authentication_param

        self.service = service
        self.request = request
        self.langage = langage or "english"
        self.allows = allows


class EPFLConfig(Config):
    """ Specific config currently used @ EPFL
    More info on config options https://tequila.epfl.ch/info """

    def __init__(self, allow_guests=False, *args, **kwargs):
        if not kwargs.get('server_url'):
            kwargs['server_url'] = "https://tequila.epfl.ch"

        super(EPFLConfig, self).__init__(*args, **kwargs)

        if not kwargs.get('org'):
            self.org = "EPFL"

        if not kwargs.get('request'):
            self.request = ["name",
                            "firstname",
                            "statut",
                            "username",
                            "classe",
                            "uniqueid",
                            "email",
                            "allunits",
                            "where",
                            "group",
                            "memberof"]

        if allow_guests:
            self.allow_guests()

    def allow_guests(self):
        """ Use preventive config, by default, set guests as 'not allowed' """
        if self.allows:
            self.allows += '|categorie=epfl-guests'
        else:
            self.allows = 'categorie=epfl-guests'
